@@ -7,11 +7,7 @@ Support for running mach tasks (via run-task)
 
 from __future__ import absolute_import, print_function, unicode_literals
 
-from taskgraph.transforms.job import run_job_using
-from taskgraph.transforms.job.run_task import (
-    docker_worker_run_task,
-    native_engine_run_task,
-)
+from taskgraph.transforms.job import run_job_using, configure_taskdesc_for_run
 from taskgraph.util.schema import Schema
 from voluptuous import Required
 
@@ -20,20 +16,20 @@ mach_schema = Schema({
 
     # The mach command (omitting `./mach`) to run
     Required('mach'): basestring,
+
+    # if true, perform a checkout of a comm-central based branch inside the
+    # gecko checkout
+    Required('comm-checkout'): bool,
 })
 
 
-@run_job_using("docker-worker", "mach", schema=mach_schema)
-@run_job_using("native-engine", "mach", schema=mach_schema)
+@run_job_using("docker-worker", "mach", schema=mach_schema, defaults={'comm-checkout': False})
+@run_job_using("native-engine", "mach", schema=mach_schema, defaults={'comm-checkout': False})
 def docker_worker_mach(config, job, taskdesc):
     run = job['run']
 
     # defer to the run_task implementation
     run['command'] = 'cd /builds/worker/checkouts/gecko && ./mach ' + run['mach']
-    run['checkout'] = True
-    run['sparse-profile'] = None
+    run['using'] = 'run-task'
     del run['mach']
-    if job['worker']['implementation'] == 'docker-worker':
-        docker_worker_run_task(config, job, taskdesc)
-    else:
-        native_engine_run_task(config, job, taskdesc)
+    configure_taskdesc_for_run(config, job, taskdesc, job['worker']['implementation'])
